;; Generated package description from typo.el  -*- no-byte-compile: t -*-
(define-package "typo" "1.0.1" "Completion style using typo analysis" '((emacs "27.1")) :commit "281c9049c1d08cf11c8a1957ea99e5c58795ed3c" :authors '(("Philip Kaludercic" . "philipk@posteo.net")) :maintainer '("Philip Kaludercic" . "~pkal/public-inbox@lists.sr.ht") :keywords '("convenience") :url "https://git.sr.ht/~pkal/typo/")
